#ifndef SGMAINWINDOW_H
#define SGMAINWINDOW_H

#include <Window.h>
#include <Font.h>
#include <Menu.h>
#include <MenuItem.h>
#include <MenuField.h>
#include <ScrollView.h>
#include <TextControl.h>
#include <TextView.h>
#include <String.h>
#include <MessageFilter.h>
#include <Messenger.h>
#include "SwordBackend.h"
#include <vector>

class FontPanel;
class SGModule;

#define M_WINDOW_CLOSED 'wcls'
using namespace std;


class EndKeyFilter : public BMessageFilter
{
public:
	EndKeyFilter(void);
	~EndKeyFilter(void);
	virtual filter_result Filter(BMessage *msg, BHandler **target);
};

class SGMainWindow : public BWindow 
{
public:
	SGMainWindow(BRect frame, const char *module, const char* key);
	~SGMainWindow();
	virtual bool QuitRequested();
	virtual void MessageReceived(BMessage *message);
	virtual void FrameResized(float width, float height);

private:
	void BuildGUI(void);
	void InsertVerseNumber(int verse);
	void InsertChapter(void);
	void LoadPrefsForModule(void);
	void SavePrefsForModule(void);
	bool NeedsLineBreaks(void);
	
	void SetModule(const TextType &module, const int32 &index);
	void SetModuleFromString(const char *name);
	void SetChapter(const int16 &chapter);

	BMenuBar		*fMenuBar;
	
	BMenuField		*fModuleField;
	
	BMenuItem		*fShowParallelItem,
					*fShowVerseNumItem,
					*fShowParallelContextItem;

	BMenu			*fBookMenu,
					*fBibleMenu,
					*fCommentaryMenu,
					*fLexiconMenu,
					*fGeneralMenu;
	
	BTextControl	*fChapterBox;
	BButton			*fNoteButton;
	
	BTextView		*fVerseView;
	BScrollView		*fScrollView;
	
	FontPanel		*fFontPanel;
	
	
	SwordBackend	*fModManager;
	SGModule		*fCurrentModule;
	uint16			fCurrentChapter;
	
	int16			fFontSize;
	BFont			*fCurrentFont;
	BFont			fDisplayFont,
					fGreekFont,
					fHebrewFont,
					fRomanFont;
	
	bool			fIsLineBreak,
					fShowVerseNumbers;
	
	BMessenger		*fFindMessenger;
};

#endif
